﻿using System;
using System.Configuration;
using System.IO;

namespace ModelFileManager
{
	public partial class RestoreDialog : Gtk.Dialog
	{
		public bool RefreshFlag = false;

		private string _BackupDirectory = "";
		private string _ModelDirectory = "";

		// ----------------------------------------------------------------------------------------------------------------------

		public RestoreDialog (string ModelDirectory, string BackupDirectory)
		{
			try
			{
				this.Build ();

				// keep locally for the restore button
				_ModelDirectory = ModelDirectory;
				_BackupDirectory = BackupDirectory;

				// the backups are named by date
				string[] subdirectoryEntries = Directory.GetDirectories(_BackupDirectory);
				foreach (string subdirectory in subdirectoryEntries)
				{
					string d = subdirectory.Substring(BackupDirectory.Length + 1);
					string backupname = String.Format("{0}-{1}-{2} {3}:{4}:{5}", d.Substring(0, 4), d.Substring(4, 2), d.Substring(6, 2), d.Substring(9, 2), d.Substring(11, 2), d.Substring(13, 2));
					cboBackupSets.AppendText(backupname);
				}

				// init Model list (wait until a backup set is selected to fill)
				Gtk.ListStore newList = new Gtk.ListStore(typeof(string));
				cboModels.Model = newList;
				cboModels.AppendText ("All");
				cboModels.Sensitive = false;

				// init Destination list
				Gtk.ListStore newList2 = new Gtk.ListStore(typeof(string));
				cboDestinations.Model = newList2;
				for (int i = 1; i <= Common.MAX_MODELS; i++)
				{
					string modelFile = System.IO.Path.Combine(ModelDirectory, String.Format("model{0}.ini", i));
					if (System.IO.File.Exists(modelFile))
					{
						string modelName = Common.GetModelName(modelFile);
						cboDestinations.AppendText(i.ToString() + " - " + modelName);
					}
				}
				cboDestinations.Sensitive = false;
				btnView.Sensitive = false;

				// init Diff funtion
				btnDiff.Sensitive = false;
			}
			catch (Exception ex) 
			{
				ModelFileManager.Common.WriteError ("RestoreDialog() constructor", ex);
			}
		}
			
		// ----------------------------------------------------------------------------------------------------------------------

		protected void OnButtonRestoreClicked (object sender, EventArgs e)
		{
			try
			{

				// first a backup set has to be selected
				string backupText = cboBackupSets.ActiveText;
				if (!String.IsNullOrEmpty(backupText)) 
				{

					// then we need to know which model or models to restore
					string modelText = cboModels.ActiveText;
					if (String.IsNullOrEmpty (modelText)) 
					{
						ModelFileManager.Common.MessageBoxInfo (this, "Must select model(s) to restore.");
					} 
					else 
					{

						// If ALL is selected...
						if (modelText == "ALL") 
						{

							// Confirm that the user wants to do this
							if (ModelFileManager.Common.MessageBoxConfirm (this, "Are you SURE you want to restore ALL MODELS from the [" + backupText + "] backup?")) 
							{

								// formulate the source and destination of the restore
								string BackupName = backupText.Substring (0, 4) + backupText.Substring (5, 2) + backupText.Substring (8, 2) + "-" + backupText.Substring (11, 2) + backupText.Substring (14, 2) + backupText.Substring (17, 2);
								string srcPath = System.IO.Path.Combine (_BackupDirectory, BackupName);
								string dstPath = _ModelDirectory;

								// continue if both paths exist
								if (Directory.Exists (srcPath) && Directory.Exists (dstPath)) 
								{
									// delete all the files from the destination
									string[] killfiles = Directory.GetFiles (dstPath);
									foreach (string killfile in killfiles)
										File.Delete (killfile);

									// copy all the new files in
									string[] newfiles = Directory.GetFiles (srcPath);
									foreach (string newfile in newfiles)
										File.Copy (newfile, System.IO.Path.Combine (dstPath, System.IO.Path.GetFileName (newfile)));
								}

								// inform the user
								ModelFileManager.Common.MessageBoxInfo (this, "Restore successful.");

								// tell main window to refresh
								RefreshFlag = true;
								this.Hide ();
							}

						} 
						else 
						{
							
							string destinationText = cboDestinations.ActiveText;
							if (!String.IsNullOrEmpty (destinationText)) 
							{
								
								string modelName = modelText.Substring (modelText.IndexOf ("-") + 1).Trim ();
								int srcModelNo = Convert.ToInt16 (modelText.Substring (0, modelText.IndexOf ("-") - 1));
								int dstModelNo = Convert.ToInt16 (destinationText.Substring (0, destinationText.IndexOf ("-") - 1));

								string BackupName = backupText.Substring (0, 4) + backupText.Substring (5, 2) + backupText.Substring (8, 2) + "-" + backupText.Substring (11, 2) + backupText.Substring (14, 2) + backupText.Substring (17, 2);
								string srcFileName = System.IO.Path.Combine (_BackupDirectory, BackupName, "model" + srcModelNo.ToString () + ".ini");
								string dstFileName = System.IO.Path.Combine (_ModelDirectory, "model" + dstModelNo.ToString () + ".ini");

								if (File.Exists (srcFileName)) 
								{
									// Confirm with the user
									if (ModelFileManager.Common.MessageBoxConfirm(this, "Are you SURE you want to restore model [" + modelName + "] from [" + backupText + "] over model slot #" + dstModelNo.ToString () + "?")) 
									{
										// Do it
										if (File.Exists (dstFileName))
											File.Delete (dstFileName);
										File.Copy (srcFileName, dstFileName);

										// Acknowledge
										ModelFileManager.Common.MessageBoxInfo (this, "Restore successful.");

										// tell main window to refresh
										RefreshFlag = true;
										this.Hide ();
									}
								} 
								else
									throw new Exception ("Unexpected Error during restore: Unable to find source file.");

							} 
							else
								ModelFileManager.Common.MessageBoxInfo (this, "Select a model slot to restore to.");

						}						
					}					

				} 
				else 
				{				
					ModelFileManager.Common.MessageBoxInfo (this, "Must select backup to restore from.");
				}
			}
			catch (Exception ex) 
			{
				ModelFileManager.Common.MessageBoxError (this, "Restore was not successful!");
				ModelFileManager.Common.WriteError ("OnButtonRestoreClicked()", ex);
			}

		}

		// ----------------------------------------------------------------------------------------------------------------------

		protected void OnButtonCancelClicked (object sender, EventArgs e)
		{
			this.Hide ();
		}

		// ----------------------------------------------------------------------------------------------------------------------

		protected void OnCboBackupSetsChanged (object sender, EventArgs e)
		{
			try
			{
				// if a backup set was selected...
				if (!String.IsNullOrEmpty(cboBackupSets.ActiveText))
				{
					// refill and disable the model dropdown list
					Gtk.ListStore newList = new Gtk.ListStore(typeof(string));
					cboModels.Model = newList;
					cboModels.AppendText ("ALL");

					string theDate = cboBackupSets.ActiveText;
					string dateDirectory = theDate.Substring(0, 4) + theDate.Substring(5, 2) + theDate.Substring(8, 2) + "-" + theDate.Substring(11, 2) + theDate.Substring(14, 2) + theDate.Substring(17, 2);
					string restorePath = System.IO.Path.Combine(_BackupDirectory, dateDirectory);

					for (int i = 1; i <= Common.MAX_MODELS; i++)
					{
						string modelFile = System.IO.Path.Combine(restorePath, String.Format("model{0}.ini", i));
						if (File.Exists(modelFile))
						{
							string modelName = Common.GetModelName(modelFile);
							if (modelName != "NONE")
								cboModels.AppendText(i.ToString() + " - " + modelName);
						}
					}
					cboModels.Sensitive = true;
					btnView.Sensitive = false;

					// disable the destination dropdown list
					cboDestinations.Sensitive = false;
					btnDiff.Sensitive = false;
				}
			}
			catch (Exception ex) 
			{
				ModelFileManager.Common.WriteError ("OnCboBackupSetsChanged()", ex);
			}
		}

		// ----------------------------------------------------------------------------------------------------------------------

		protected void OnCboModelsChanged (object sender, EventArgs e)
		{
			if (!String.IsNullOrEmpty(cboModels.ActiveText))
			{
				if (cboModels.ActiveText == "ALL") // all selected
				{
					cboDestinations.Sensitive = false;
					btnView.Sensitive = false;
					btnDiff.Sensitive = false;
				}
				else
				{
					cboDestinations.Sensitive = true;
					btnView.Sensitive = true;
				}
			}
		}

		// ----------------------------------------------------------------------------------------------------------------------

		protected void OnCboDestinationsChanged (object sender, EventArgs e)
		{
			btnDiff.Sensitive = (!String.IsNullOrEmpty(cboDestinations.ActiveText));
		}

		// ----------------------------------------------------------------------------------------------------------------------

		protected void OnBtnViewClicked (object sender, EventArgs e)
		{
			try
			{
				if (!String.IsNullOrEmpty(cboModels.ActiveText) & cboModels.ActiveText != "ALL")   // skip the first
				{
					string srcModelItem = cboModels.ActiveText;
					//string modelName = srcModelItem.Substring(srcModelItem.IndexOf("-") + 1).Trim();
					int srcModelNo = Convert.ToInt16(srcModelItem.Substring(0, srcModelItem.IndexOf("-") - 1));

					string backupText = cboBackupSets.ActiveText;
					string BackupName = backupText.Substring(0, 4) + backupText.Substring(5, 2) + backupText.Substring(8, 2) + "-" + backupText.Substring(11, 2) + backupText.Substring(14, 2) + backupText.Substring(17, 2);
					string srcFileName = System.IO.Path.Combine(_BackupDirectory, BackupName, "model" + srcModelNo.ToString() + ".ini");

					if (File.Exists(srcFileName))
					{
						string exeFile = ConfigurationManager.AppSettings["BackupViewer"];
						if (exeFile == "") 
						{
							ModelFileManager.ViewWindow viewer = new ModelFileManager.ViewWindow(srcFileName);
							viewer.Show();
						}
						else
						{
							string argTemplate = ConfigurationManager.AppSettings["BackupViewerArgs"];
							if (argTemplate == "")
								argTemplate = "{0}";
							string args = String.Format(argTemplate, "\"" + srcFileName + "\"");

							System.Diagnostics.ProcessStartInfo startInfo = new System.Diagnostics.ProcessStartInfo();
							startInfo.FileName = exeFile;
							startInfo.Arguments = args;
							System.Diagnostics.Process.Start(startInfo);
						}
					}
				}
				else
					ModelFileManager.Common.MessageBoxInfo(this, "Select a model from the backup set.");
			}
			catch (Exception ex) 
			{
				ModelFileManager.Common.WriteError("OnBtnViewClicked()", ex);
			}
		}


		// ----------------------------------------------------------------------------------------------------------------------

		protected void OnBtnDiffClicked (object sender, EventArgs e)
		{
			try
			{
				if (!String.IsNullOrEmpty(cboDestinations.ActiveText))
				{
					string srcModelItem = cboModels.ActiveText;
					//string modelName = srcModelItem.Substring(srcModelItem.IndexOf("-") + 1).Trim();
					int srcModelNo = Convert.ToInt16(srcModelItem.Substring(0, srcModelItem.IndexOf("-") - 1));

					string dstModel = cboDestinations.ActiveText;
					int dstModelNo = Convert.ToInt16(dstModel.Substring(0, dstModel.IndexOf("-") - 1));

					string backupText = cboBackupSets.ActiveText;
					string BackupName = backupText.Substring(0, 4) + backupText.Substring(5, 2) + backupText.Substring(8, 2) + "-" + backupText.Substring(11, 2) + backupText.Substring(14, 2) + backupText.Substring(17, 2);
					string srcFileName = System.IO.Path.Combine(_BackupDirectory, BackupName, "model" + srcModelNo.ToString() + ".ini");
					string dstFileName = System.IO.Path.Combine(_ModelDirectory, "model" + dstModelNo.ToString() + ".ini");

					if (System.IO.File.Exists(srcFileName) && System.IO.File.Exists(dstFileName))
					{

						string exeFile = ConfigurationManager.AppSettings["DiffViewer"];
						if (exeFile == "")
						{
							ModelFileManager.Common.MessageBoxInfo(this, "Please set the [DiffViewer] setting in the ModelFileManager.exe.config file.");
						}
						else
						{                            
							string argTemplate = ConfigurationManager.AppSettings["DiffViewerArgs"];
							if (argTemplate == "")
								argTemplate = "{0} {1}";
							string args = String.Format(argTemplate, "\"" + srcFileName + "\"", "\"" + dstFileName + "\"");

							System.Diagnostics.ProcessStartInfo startInfo = new System.Diagnostics.ProcessStartInfo();
							startInfo.FileName = exeFile;
							startInfo.Arguments = args;
							System.Diagnostics.Process.Start(startInfo);
						}
					}
				}
			}
			catch (Exception ex)
			{
				ModelFileManager.Common.WriteError("OnBtnDiffClicked()", ex);
			}

		}

	}
}

